#include "id3ed.h"

static char *id3v1_genre[] = {
	"Blues",
        "Classic Rock",
        "Country",
        "Dance",
        "Disco",
        "Funk",
        "Grunge",
        "Hip-Hop",
        "Jazz",
        "Metal",
        "New Age",
        "Oldies",
        "Other",
        "Pop",
        "R&B",
        "Rap",
        "Reggae",
        "Rock",
        "Techno",
        "Industrial",
        "Alternative",
        "Ska",
        "Death Metal",
        "Pranks",
        "Soundtrack",
        "Euro-Techno",
        "Ambient",
        "Trip-Hop",
        "Vocal",
        "Jazz+Funk",
        "Fusion",
        "Trance",
        "Classical",
        "Instrumental",
        "Acid",
        "House",
        "Game",
        "Sound Clip",
        "Gospel",
        "Noise",
        "AlternRock",
        "Bass",
        "Soul",
        "Punk",
        "Space",
        "Meditative",
        "Instrumental Pop",
        "Instrumental Rock",
        "Ethnic",
        "Gothic",
        "Darkwave",
        "Techno-Industrial",
        "Electronic",
        "Pop-Folk",
        "Eurodance",
        "Dream",
        "Southern Rock",
        "Comedy",
        "Cult",
        "Gangsta",
        "Top 40",
        "Christian Rap",
        "Pop/Funk",
        "Jungle",
        "Native American",
        "Cabaret",
        "New Wave",
        "Psychadelic",
        "Rave",
        "Showtunes",
        "Trailer",
        "Lo-Fi",
        "Tribal",
        "Acid Punk",
        "Acid Jazz",
        "Polka",
        "Retro",
        "Musical",
        "Rock & Roll",
        "Hard Rock",
        "Folk",
        "Folk-Rock",
        "National Folk",
        "Swing",
        "Fast Fusion",
        "Bebob",
        "Latin",
        "Revival",
        "Celtic",
        "Bluegrass",
        "Avantgarde",
        "Gothic Rock",
        "Progressive Rock",
        "Psychedelic Rock",
        "Symphonic Rock",
        "Slow Rock",
        "Big Band",
        "Chorus",
        "Easy Listening",
        "Acoustic",
        "Humour",
        "Speech",
        "Chanson",
        "Opera",
        "Chamber Music",
        "Sonata",
        "Symphony",
        "Booty Bass",
        "Primus",
        "Porn Groove",
        "Satire",
        "Slow Jam",
        "Club",
        "Tango",
        "Samba",
        "Folklore",
        "Ballad",
        "Power Ballad",
        "Rhythmic Soul",
        "Freestyle",
        "Duet",
        "Punk Rock",
        "Drum Solo",
        "A capella",
        "Euro-House",
        "Dance Hall"
};

size_t id3v1_genre_count = sizeof(id3v1_genre)/sizeof(id3v1_genre[0]);

struct id3v1_tab {
	char *name;
	int ind;
	size_t len;
};

struct id3v1_tab id3v1_tab[128];

static int
id3v1_tab_cmp(const void *a, const void *b)
{
	const struct id3v1_tab *ap = a;
	const struct id3v1_tab *bp = b;
	return strcasecmp(ap->name, bp->name);
}

static int
id3v1_fill_tab()
{
	int i;

	for (i = 0; i < id3v1_genre_count; i++) {
		id3v1_tab[i].name = id3v1_genre[i];
		id3v1_tab[i].ind = i;
		id3v1_tab[i].len = strlen(id3v1_tab[i].name);
	}
	qsort(id3v1_tab, id3v1_genre_count, sizeof(id3v1_tab[0]),
	      id3v1_tab_cmp);
}

const char *
id3v1_n_to_genre(unsigned n)
{
	if (n < id3v1_genre_count)
		return id3v1_genre[n];
	return "Unknown";
}

int
id3v1_genre_to_n(const char *genre)
{
	int i, index = -1;
	int ambiguous;
	size_t glen = strlen(genre);
	if (!id3v1_tab[0].name)
		id3v1_fill_tab();		
	for (i = 0; i < id3v1_genre_count; i++) {
		if (glen <= id3v1_tab[i].len
		    && strncasecmp(id3v1_tab[i].name, genre, glen) == 0) {
			if (glen == id3v1_tab[i].len) 
				return id3v1_tab[i].ind;
			else if (index == -1)
				index = id3v1_tab[i].ind;
			else
				return -2;
		} else if (index != -1 && genre[0] != id3v1_tab[i].name[0])
			break;
	}
	return index;
}
		
		

	
